#include "HiResTimer.h"

HiResTimer::HiResTimer()
{
#ifdef _WIN32
	QueryPerformanceFrequency(&m_freq);
#elif __APPLE__
	m_conversion = 0.0;
	mach_timebase_info_data_t info;
	kern_return_t err = mach_timebase_info(&info);
	if (err == 0)
	  m_conversion = 1e-9 * (double) info.numer / (double) info.denom;
#else
#endif
	Reset();
}

HiResTimer::~HiResTimer()
{
}

void HiResTimer::Reset()
{
#ifdef _WIN32
	QueryPerformanceCounter(&m_startTime);
#elif __APPLE__
	m_startTime = mach_absolute_time();
#else
	clock_gettime(CLOCK_MONOTONIC, &m_startTime); 
#endif
}

double HiResTimer::GetElapsed()
{
#ifdef _WIN32  
	LARGE_INTEGER endTime;
	QueryPerformanceCounter(&endTime);

	return (double) (endTime.QuadPart - m_startTime.QuadPart) / (double) m_freq.QuadPart;
#elif __APPLE__
	uint64_t stop = mach_absolute_time();
	return m_conversion * ((double) (mach_absolute_time() - m_startTime));
#else
	timespec endTime;
	int result = clock_gettime(CLOCK_MONOTONIC, &endTime); 
	long sec  = endTime.tv_sec - m_startTime.tv_sec;
	long nano = endTime.tv_nsec - m_startTime.tv_nsec;
	return sec + (nano / (double) 1e9);
#endif
}

