// Socket
// Base class for our SocketClient and SocketServer classes.
// Contains all the shared functionality of both clients and servers.
// Also this is the class instantiated when a SocketServer accepts a connection.

#ifndef __SOCKET_H__
#define __SOCKET_H__

#ifdef _WIN32
#include <winsock2.h>
#include <ws2tcpip.h>
#else
#include <netinet/in.h>
#include <netdb.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <netinet/tcp.h>

// Declare a few things that Windows has but not others
#define INVALID_SOCKET -1
typedef int SOCKET;
#endif

#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "Threading.h"

using namespace std;

// Structure for pass back info about our communication peer
struct PeerInfo 
{
	char name[INET6_ADDRSTRLEN];
	int  port;
};

class Socket
{
	public:
		virtual			~Socket();
		int				SendBytes(char* buffer, int len);
		int				RecvBytes(char* buffer, int maxlen);
		void			Close();
		PeerInfo		GetPeerInfo();

		// Bit options for the flags integer passed when in on creating a socket
		const static int NO_DELAY   = 1;	// Use the TCP_NODELAY option to disable Nagle's algorithm
		const static int KEEP_ALIVE = 2;	// Enable TCP keep-alives
		const static int DATAGRAM   = 4;	// Connection-oriented datagram UDP transport instead of TCP

	protected:
		friend class SocketServer;		

		Socket(int flags);
		Socket(SOCKET sock, int flags);
		void			SetOptions();

		int				m_flags;		// Stores flags that change behavior
		SOCKET			m_sock;			// Socket file descriptor


	private:
		void			Startup();
		void			Shutdown();
		
		static mutex_t  m_mutex;
		static int		m_numSockets;		// Tracks how many Socket objects exist
};

#endif
