
#include "SocketClient.h"

// Construct a new socket client, opening a connection
//  host   - name of the server to connect to
//  port   - port number to connect on
//  flags  - various options, see Socket.h
//  good   - return true on succes, false otherwise
SocketClient::SocketClient(const char* host, int port, int flags, bool& good) : Socket(flags)
{	
	struct addrinfo hints;
	struct addrinfo *res = NULL;		// will point to the results
	int status;
	
	good = false;

	memset(&hints, 0, sizeof hints);	// make sure the struct is empty
	hints.ai_family = AF_INET;			// specify IPv4 
	if (flags & DATAGRAM)
		hints.ai_socktype = SOCK_DGRAM;		// UDP datagram service
	else
		hints.ai_socktype = SOCK_STREAM;	// TCP stream service

	// Get ready to connect
	char portNum[16];
#ifdef _WIN32
	sprintf_s(portNum, "%d", port);
#else
	sprintf(portNum, "%d", port);
#endif
	status = getaddrinfo(host, portNum, &hints, &res);

	if (status != 0)
	{
		cerr << "SocketClient::SocketClient, getaddrinfo failed" << endl;
		return;
	}

	// Open up the socket
	m_sock = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
	if (m_sock == INVALID_SOCKET) 
	{
		cerr << "SocketClient::SocketClient, socket failed" << endl;
		freeaddrinfo(res);
		return;
	}
	
	// Make the connection with the server
	if (connect(m_sock, res->ai_addr, res->ai_addrlen) == INVALID_SOCKET)
	{
		m_sock = INVALID_SOCKET;
		cerr << "SocketClient::SocketClient, connect failed" << endl;
		freeaddrinfo(res);
		return;
	}

	SetOptions();

	freeaddrinfo(res);
	good = true;
}


