
#include "SocketServer.h"

// Construct a new object whose job it is to wait for new clients
// to arrive and then hand off a new Socket object to take care
// of their requests.
//   port        - port number we accept connections on
//   maxWaiting  - maximum number of clients that will queue up waiting for an accept
//   flags       - TBD
//   good		 - return true on succes, false otherwise
SocketServer::SocketServer(int port, int maxWaiting, int flags, bool& good) : Socket(flags)
{
	int status;
	struct addrinfo hints;
	struct addrinfo *servinfo;			// will point to the results

	good = false;

	memset(&hints, 0, sizeof hints);	// make sure the struct is empty
	hints.ai_family   = AF_INET;        // specify IPv4 
	hints.ai_flags    = AI_PASSIVE;     // fill in my IP for me	
	hints.ai_socktype = SOCK_STREAM;	// TCP stream service

	char portNum[16];
#ifdef _WIN32
	sprintf_s(portNum, "%d", port);
#else
	sprintf(portNum, "%d", port);
#endif

	status = getaddrinfo(NULL, portNum, &hints, &servinfo);	
	if (status != 0)
	{
	    cerr << "SocketServer::SocketServer, getaddrinfo failed" << endl;
	    return;
	}
	m_sock = socket(servinfo->ai_family, servinfo->ai_socktype, servinfo->ai_protocol);

	// Set some options on the socket
	SetOptions();

	if (bind(m_sock, servinfo->ai_addr, servinfo->ai_addrlen) == INVALID_SOCKET)
	{
		m_sock = INVALID_SOCKET;
		cerr << "SocketServer::SocketServer, bind failed" << endl;
		return;
	}

	freeaddrinfo(servinfo); // free the linked-list

	cout << "listen, maxWaiting = " << maxWaiting << endl;

	if (listen(m_sock, maxWaiting) == INVALID_SOCKET) 
	{
		cerr << "SocketServer::SocketServer, listen failed" << endl;
		return;
	}

	good = true;
}

// Accept a new client connection, returns a new Socket object for the connection.
// Returns NULL on error.
Socket* SocketServer::Accept()
{
	struct sockaddr_storage their_addr;
	socklen_t addr_size;

    addr_size = sizeof their_addr;
    SOCKET sock = accept(m_sock, (struct sockaddr *)&their_addr, &addr_size);
	
	if (sock == INVALID_SOCKET)
	{
		cerr << "SocketServer::Accept, accept failed" << endl;
		return NULL;
	}

	return new Socket(sock, m_flags);
}

