
#include "Utils.h"

// Change a STL string to lower case
string makeLower(const string& strA)
{
	string strLowerA = "";
	unsigned int i = 0;
	unsigned int iLenA = (unsigned int) strA.length();

	for (i = 0; i < iLenA; i++)
	{
	  strLowerA += tolower(strA[i]);
	}

	return strLowerA;
}

// Change a STL string to upper case
string makeUpper(const string& strA)
{
	string strUpperA = "";
	unsigned int i = 0;
	unsigned int iLenA = (unsigned int) strA.length();

	for (i = 0; i < iLenA; i++)
	{
	  strUpperA += toupper(strA[i]);
	}

	return strUpperA;
}

// Checks for a command line switch with no following argument
bool getSwitch(int argc, char** argv, const string& strSwitch)
{
	string strLowerSwitch = makeLower(strSwitch);
	for (int i = 1; i < argc; i++)
	{
		string strArg = makeLower(argv[i]);
		if ((strArg.find(strLowerSwitch) == 1) && 
			((strArg[0] == '/') || (strArg[0] == '-')))
		{
			return true;
		}
	}
	return false;
}

// Look for a command line switch with the given name and followed by a single string parameter
bool getSwitch(int argc, char** argv, const string& strSwitch, string* pResult)
{
	if (!pResult)
		return false;
	string strLowerSwitch = makeLower(strSwitch);
	for (int i = 1; i < argc - 1; i++)
	{
		string strArg = makeLower(argv[i]);
		if ((strArg.find(strLowerSwitch) == 1) && 
			((strArg[0] == '/') || (strArg[0] == '-')))
		{
			*pResult = argv[i + 1];
			return true;
		}
	}
	return false;
}

// Look for a command line switch with the given name and followed by one integer parameters
bool getSwitch(int argc, char** argv, const string& strSwitch, int* pResult1)
{
	if (!pResult1) 
		return false;
	string strLowerSwitch = makeLower(strSwitch);
	for (int i = 1; i < argc - 1; i++)
	{
		string strArg = makeLower(argv[i]);
		if ((strArg.find(strLowerSwitch) == 1) && 
			((strArg[0] == '/') || (strArg[0] == '-')))
		{
			*pResult1 = atoi(argv[i + 1]);
			return true;
		}
	}
	return false;
}

// Look for a command line switch with the given name and followed by one integer parameters
bool getSwitch(int argc, char** argv, const string& strSwitch, unsigned int* pResult1)
{
	if (!pResult1) 
		return false;
	string strLowerSwitch = makeLower(strSwitch);
	for (int i = 1; i < argc - 1; i++)
	{
		string strArg = makeLower(argv[i]);
		if ((strArg.find(strLowerSwitch) == 1) && 
			((strArg[0] == '/') || (strArg[0] == '-')))
		{
			*pResult1 = atoi(argv[i + 1]);
			return true;
		}
	}
	return false;
}

// Look for a command line switch with the given name and followed by two integer parameters
bool getSwitch(int argc, char** argv, const string& strSwitch, int* pResult1, int* pResult2)
{
	if ((!pResult1) || (!pResult2))
		return false;
	string strLowerSwitch = makeLower(strSwitch);
	for (int i = 1; i < argc - 2; i++)
	{
		string strArg = makeLower(argv[i]);
		if ((strArg.find(strLowerSwitch) == 1) && 
			((strArg[0] == '/') || (strArg[0] == '-')))
		{
			*pResult1 = atoi(argv[i + 1]);
			*pResult2 = atoi(argv[i + 2]);
			return true;
		}
	}
	return false;
}

#ifdef _WIN32
unsigned char getKeyAsync() 
{
	if (!_kbhit())
		return 0;
	return _getch();
}

unsigned char getKey() 
{
	while (!_kbhit())
		Sleep(20);
	return _getch();
}

#else

// Non-blocking return of a keyboard character
// http://www.pwilson.net/kbhit.html

extern int errno;                 
static struct termios termattr, save_termattr;
static int ttysavefd = -1;
static enum 
{ 
  RESET, RAW, CBREAK 
} ttystate = RESET;

// set_tty_raw(), put the user's TTY in one-character-at-a-time mode.
// returns 0 on success, -1 on failure.
int set_tty_raw() 
{
  int i;

  i = tcgetattr (STDIN_FILENO, &termattr);
  if (i < 0) 
  {
	printf("tcgetattr() returned %d for fildes=%d\n",i,STDIN_FILENO); 
	perror ("");
	return -1;
  }
  save_termattr = termattr;

  termattr.c_lflag &= ~(ECHO | ICANON | IEXTEN | ISIG);
  termattr.c_iflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP | IXON);
  termattr.c_cflag &= ~(CSIZE | PARENB);
  termattr.c_cflag |= CS8;
  termattr.c_oflag &= ~(OPOST);
   
  termattr.c_cc[VMIN] = 1;  /* or 0 for some Unices;  see note 1 */
  termattr.c_cc[VTIME] = 0;

  i = tcsetattr (STDIN_FILENO, TCSANOW, &termattr);
  if (i < 0) 
  {
	printf("tcsetattr() returned %d for fildes=%d\n",i,STDIN_FILENO); 
	perror("");
	return -1;
  }
   
  ttystate = RAW;
  ttysavefd = STDIN_FILENO;

  return 0;
}

// set_tty_cbreak(), put the user's TTY in cbreak mode.
// returns 0 on success, -1 on failure.
int  set_tty_cbreak() 
{
  int i;

  i = tcgetattr (STDIN_FILENO, &termattr);
  if (i < 0) 
  {
	printf("tcgetattr() returned %d for fildes=%d\n",i,STDIN_FILENO); 
	perror ("");
	return -1;
  }

  save_termattr = termattr;

  termattr.c_lflag &= ~(ECHO | ICANON);
  termattr.c_cc[VMIN] = 1;
  termattr.c_cc[VTIME] = 0;
	  
  i = tcsetattr (STDIN_FILENO, TCSANOW, &termattr);
  if (i < 0) 
  {
	printf("tcsetattr() returned %d for fildes=%d\n",i,STDIN_FILENO); 
	perror ("");
	return -1;
  }
  ttystate = CBREAK;
  ttysavefd = STDIN_FILENO;

  return 0;
}

// set_tty_cooked(), restore normal TTY mode. Very important to call
// the function before exiting else the TTY won't be too usable.
// returns 0 on success, -1 on failure.
int set_tty_cooked() 
{
  int i;
  if (ttystate != CBREAK && ttystate != RAW) 
  {
	return 0;
  }
  i = tcsetattr (STDIN_FILENO, TCSAFLUSH, &save_termattr);
  if (i < 0) 
  {
	return -1;
  }
  ttystate = RESET;
  return 0;
}

// getKeyAsync(), if there's a typed character waiting to be read,
// return it; else return 0.
// 10-sep-2006: kb_getc() fails (it hangs on the read() and never returns
// until a char is typed) under some Unix/Linux versions: ubuntu, suse, and
// maybe others. To make it work, please uncomment two source lines below.
unsigned char getKeyAsync() 
{
  int i;
  unsigned char ch;
  ssize_t size;
  
  set_tty_raw();

  termattr.c_cc[VMIN] = 0;  /* uncomment if needed */
  i = tcsetattr (STDIN_FILENO, TCSANOW, &termattr);
  size = read (STDIN_FILENO, &ch, 1);
  termattr.c_cc[VMIN] = 1;  /* uncomment if needed */
  i = tcsetattr (STDIN_FILENO, TCSANOW, &termattr);
  
  set_tty_cooked();

  if (size == 0)
  {
	return 0;
  }
  else
  {
	return ch;
  }
}

// getKey, wait for a character to be typed and return it.
unsigned char getKey() 
{
  unsigned char ch;
  size_t size;

  while (1)
  {
	usleep(20000);        /* 1/50th second: thanks, Floyd! */
	size = read (STDIN_FILENO, &ch, 1);
	if (size > 0)
	{
	  break;
	}
  }
  return ch;
}

#endif 

// Gets the time and/or date stamp 
string getTimeDateStamp(bool date, bool time)
{
    std::string strTimeStamp = "";

    struct timeb timebuffer;
    char* timeline = NULL;

    ftime(&timebuffer);
    timeline = ctime( & ( timebuffer.time ) );

    // Format is:
    // Wed Jun 22 10:22:00 2005
    // 0123456789012345678901234
    if ((timeline != NULL) && (strlen(timeline) > 23))
    {
        if (date)
        {
            for (int i = 4; i < 10; i++)
                strTimeStamp += timeline[i];            
            for (int i = 19; i < 24; i++)
                strTimeStamp += timeline[i];            
            if (time)
                strTimeStamp += " ";
        }

        if (time)
        {
            for (int i = 11; i < 19; i++)
                strTimeStamp += timeline[i];
            strTimeStamp += ".";
            char strMs[16];
#ifdef _WIN32
			sprintf_s(strMs, "%d", timebuffer.millitm);
#else
            sprintf(strMs, "%d", timebuffer.millitm);
#endif

            if (strlen(strMs) == 1)
                strTimeStamp += "00";
            else if (strlen(strMs) == 2)
                strTimeStamp += "0";
            strTimeStamp += strMs;
        }                
    }
    return strTimeStamp;
}